@@ -16,14 +16,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef G10_KEYDB_H
 #define G10_KEYDB_H
 
-#include "types.h"
-#include "util.h"
+#include "../common/types.h"
+#include "../common/util.h"
 #include "packet.h"
 
 /* What qualifies as a certification (rather than a signature?) */
@@ -61,12 +61,6 @@ struct kbnode_struct {
 #define is_cloned_kbnode(a)   ((a)->private_flag & 2)
 
 
-enum resource_type {
-    rt_UNKNOWN = 0,
-    rt_RING = 1
-};
-
-
 /* Bit flags used with build_pk_list.  */
 enum
   {
@@ -75,28 +69,14 @@ enum
     PK_LIST_CONFIG     = 4, /* Specified via config file.          */
     PK_LIST_FROM_FILE  = 8  /* Take key from file with that name.  */
   };
+
 /* To store private data in the flags the private data must be left
-   shifted by this value.  */
+ * shifted by this value.  */
 enum
   {
     PK_LIST_SHIFT = 4
   };
 
-/****************
- * A data structure to hold information about the external position
- * of a keyblock.
- */
-struct keyblock_pos_struct {
-    int   resno;     /* resource number */
-    enum resource_type rt;
-    off_t offset;    /* position information */
-    unsigned count;  /* length of the keyblock in packets */
-    iobuf_t  fp;     /* Used by enum_keyblocks. */
-    int secret;      /* working on a secret keyring */
-    PACKET *pkt;     /* ditto */
-    int valid;
-};
-typedef struct keyblock_pos_struct KBPOS;
 
 /* Structure to hold a couple of public key certificates. */
 typedef struct pk_list *PK_LIST;  /* Deprecated. */
@@ -129,9 +109,6 @@ struct pubkey_find_info {
 };
 
 
-typedef struct keydb_handle *KEYDB_HANDLE;
-
-
 /* Helper type for preference fucntions. */
 union pref_hint
 {
@@ -139,6 +116,19 @@ union pref_hint
 };
 
 
+/* Constants to describe from where a key was fetched or updated.  */
+enum
+  {
+    KEYSRC_UNKNOWN = 0,
+    KEYSRC_FILE    = 1, /* Direct import from a file.  */
+    KEYSRC_KS      = 2, /* Public keyserver.    */
+    KEYSRC_PREF_KS = 3, /* Preferred keysrver.  */
+    KEYSRC_WKD     = 4, /* Web Key Directory.   */
+    KEYSRC_WKD_SD  = 5, /* Web Key Directory but from a sub domain.  */
+    KEYSRC_DANE    = 6  /* OpenPGP DANE.  */
+  };
+
+
 /*-- keydb.c --*/
 
 #define KEYDB_RESOURCE_FLAG_PRIMARY  2  /* The primary resource.  */
@@ -181,7 +171,7 @@ const char *keydb_get_resource_name (KEYDB_HANDLE hd);
 gpg_error_t keydb_get_keyblock (KEYDB_HANDLE hd, KBNODE *ret_kb);
 
 /* Update the keyblock KB.  */
-gpg_error_t keydb_update_keyblock (KEYDB_HANDLE hd, kbnode_t kb);
+gpg_error_t keydb_update_keyblock (ctrl_t ctrl, KEYDB_HANDLE hd, kbnode_t kb);
 
 /* Insert a keyblock into one of the underlying keyrings or keyboxes.  */
 gpg_error_t keydb_insert_keyblock (KEYDB_HANDLE hd, kbnode_t kb);
@@ -193,7 +183,7 @@ gpg_error_t keydb_delete_keyblock (KEYDB_HANDLE hd);
 gpg_error_t keydb_locate_writable (KEYDB_HANDLE hd);
 
 /* Rebuild the on-disk caches of all key resources.  */
-void keydb_rebuild_caches (int noisy);
+void keydb_rebuild_caches (ctrl_t ctrl, int noisy);
 
 /* Return the number of skipped blocks (because they were to large to
    read from a keybox) since the last search reset.  */
@@ -222,7 +212,7 @@ gpg_error_t keydb_search_fpr (KEYDB_HANDLE hd, const byte *fpr);
 
 
 /*-- pkclist.c --*/
-void show_revocation_reason( PKT_public_key *pk, int mode );
+void show_revocation_reason (ctrl_t ctrl, PKT_public_key *pk, int mode );
 int  check_signatures_trust (ctrl_t ctrl, PKT_signature *sig);
 
 void release_pk_list (PK_LIST pk_list);
@@ -252,26 +242,28 @@ int  have_static_passphrase(void);
 const char *get_static_passphrase (void);
 void set_passphrase_from_string(const char *pass);
 void read_passphrase_from_fd( int fd );
-void passphrase_clear_cache ( u32 *keyid, const char *cacheid, int algo );
+void passphrase_clear_cache (const char *cacheid);
 DEK *passphrase_to_dek_ext(u32 *keyid, int pubkey_algo,
                            int cipher_algo, STRING2KEY *s2k, int mode,
                            const char *tryagain_text,
                            const char *custdesc, const char *custprompt,
                            int *canceled);
-DEK *passphrase_to_dek( u32 *keyid, int pubkey_algo,
-			int cipher_algo, STRING2KEY *s2k, int mode,
+DEK *passphrase_to_dek (int cipher_algo, STRING2KEY *s2k,
+                        int create, int nocache,
                         const char *tryagain_text, int *canceled);
 void set_next_passphrase( const char *s );
 char *get_last_passphrase(void);
 void next_to_last_passphrase(void);
 
-void emit_status_need_passphrase (u32 *keyid, u32 *mainkeyid, int pubkey_algo);
+void emit_status_need_passphrase (ctrl_t ctrl, u32 *keyid,
+                                  u32 *mainkeyid, int pubkey_algo);
 
 #define FORMAT_KEYDESC_NORMAL  0
 #define FORMAT_KEYDESC_IMPORT  1
 #define FORMAT_KEYDESC_EXPORT  2
 #define FORMAT_KEYDESC_DELKEY  3
-char *gpg_format_keydesc (PKT_public_key *pk, int mode, int escaped);
+char *gpg_format_keydesc (ctrl_t ctrl,
+                          PKT_public_key *pk, int mode, int escaped);
 
 
 /*-- getkey.c --*/
@@ -283,7 +275,7 @@ void cache_public_key( PKT_public_key *pk );
 void getkey_disable_caches(void);
 
 /* Return the public key with the key id KEYID and store it at PK.  */
-int get_pubkey( PKT_public_key *pk, u32 *keyid );
+int get_pubkey (ctrl_t ctrl, PKT_public_key *pk, u32 *keyid);
 
 /* Similar to get_pubkey, but it does not take PK->REQ_USAGE into
    account nor does it merge in the self-signed data.  This function
@@ -291,7 +283,7 @@ int get_pubkey( PKT_public_key *pk, u32 *keyid );
 int get_pubkey_fast (PKT_public_key *pk, u32 *keyid);
 
 /* Return the key block for the key with KEYID.  */
-kbnode_t get_pubkeyblock (u32 *keyid);
+kbnode_t get_pubkeyblock (ctrl_t ctrl, u32 *keyid);
 
 /* A list used by get_pubkeys to gather all of the matches.  */
 struct pubkey_s
@@ -324,16 +316,23 @@ int get_pubkey_byname (ctrl_t ctrl,
                        KBNODE *ret_keyblock, KEYDB_HANDLE *ret_kdbhd,
 		       int include_unusable, int no_akl );
 
+/* Likewise, but only return the best match if NAME resembles a mail
+ * address.  */
+int get_best_pubkey_byname (ctrl_t ctrl,
+			    GETKEY_CTX *retctx, PKT_public_key *pk,
+			    const char *name, KBNODE *ret_keyblock,
+			    int include_unusable, int no_akl);
+
 /* Get a public key directly from file FNAME.  */
 gpg_error_t get_pubkey_fromfile (ctrl_t ctrl,
                                  PKT_public_key *pk, const char *fname);
 
 /* Return the public key with the key id KEYID iff the secret key is
  * available and store it at PK.  */
-gpg_error_t get_seckey (PKT_public_key *pk, u32 *keyid);
+gpg_error_t get_seckey (ctrl_t ctrl, PKT_public_key *pk, u32 *keyid);
 
 /* Lookup a key with the specified fingerprint.  */
-int get_pubkey_byfprint (PKT_public_key *pk,  kbnode_t *r_keyblock,
+int get_pubkey_byfprint (ctrl_t ctrl, PKT_public_key *pk, kbnode_t *r_keyblock,
                          const byte *fprint, size_t fprint_len);
 
 /* This function is similar to get_pubkey_byfprint, but it doesn't
@@ -354,7 +353,8 @@ const char *parse_def_secret_key (ctrl_t ctrl);
 gpg_error_t get_seckey_default (ctrl_t ctrl, PKT_public_key *pk);
 
 /* Search for keys matching some criteria.  */
-gpg_error_t getkey_bynames (getkey_ctx_t *retctx, PKT_public_key *pk,
+gpg_error_t getkey_bynames (ctrl_t ctrl,
+                            getkey_ctx_t *retctx, PKT_public_key *pk,
                             strlist_t names, int want_secret,
                             kbnode_t *ret_keyblock);
 
@@ -365,11 +365,11 @@ gpg_error_t getkey_byname (ctrl_t ctrl,
                            kbnode_t *ret_keyblock);
 
 /* Return the next search result.  */
-gpg_error_t getkey_next (getkey_ctx_t ctx, PKT_public_key *pk,
-                         kbnode_t *ret_keyblock);
+gpg_error_t getkey_next (ctrl_t ctrl, getkey_ctx_t ctx,
+                         PKT_public_key *pk, kbnode_t *ret_keyblock);
 
 /* Release any resources used by a key listing context.  */
-void getkey_end (getkey_ctx_t ctx);
+void getkey_end (ctrl_t ctrl, getkey_ctx_t ctx);
 
 /* Return the database handle used by this context.  The context still
    owns the handle.  */
@@ -383,14 +383,14 @@ void setup_main_keyids (kbnode_t keyblock);
 
 /* This function merges information from the self-signed data into the
    data structures.  */
-void merge_keys_and_selfsig (kbnode_t keyblock);
+void merge_keys_and_selfsig (ctrl_t ctrl, kbnode_t keyblock);
 
-char*get_user_id_string_native( u32 *keyid );
-char*get_long_user_id_string( u32 *keyid );
-char*get_user_id( u32 *keyid, size_t *rn );
-char*get_user_id_native( u32 *keyid );
-char *get_user_id_byfpr (const byte *fpr, size_t *rn);
-char *get_user_id_byfpr_native (const byte *fpr);
+char*get_user_id_string_native (ctrl_t ctrl, u32 *keyid);
+char*get_long_user_id_string (ctrl_t ctrl, u32 *keyid);
+char*get_user_id (ctrl_t ctrl, u32 *keyid, size_t *rn);
+char*get_user_id_native (ctrl_t ctrl, u32 *keyid);
+char *get_user_id_byfpr (ctrl_t ctrl, const byte *fpr, size_t *rn);
+char *get_user_id_byfpr_native (ctrl_t ctrl, const byte *fpr);
 
 void release_akl(void);
 int parse_auto_key_locate(char *options);
@@ -426,6 +426,13 @@ keyid_cmp (const u32 *a, const u32 *b)
   return 0;
 }
 
+/* Return whether PK is a primary key.  */
+static int GPGRT_ATTR_UNUSED
+pk_is_primary (PKT_public_key *pk)
+{
+  return keyid_cmp (pk_keyid (pk), pk_main_keyid (pk)) == 0;
+}
+
 /* Copy the keyid in SRC to DEST and return DEST.  */
 u32 *keyid_copy (u32 *dest, const u32 *src);
 
@@ -442,8 +449,9 @@ const char *pk_keyid_str (PKT_public_key *pk);
 
 const char *keystr_from_desc(KEYDB_SEARCH_DESC *desc);
 u32 keyid_from_pk( PKT_public_key *pk, u32 *keyid );
-u32 keyid_from_sig( PKT_signature *sig, u32 *keyid );
-u32 keyid_from_fingerprint(const byte *fprint, size_t fprint_len, u32 *keyid);
+u32 keyid_from_sig (PKT_signature *sig, u32 *keyid );
+u32 keyid_from_fingerprint (ctrl_t ctrl, const byte *fprint, size_t fprint_len,
+                            u32 *keyid);
 byte *namehash_from_uid(PKT_user_id *uid);
 unsigned nbits_from_pk( PKT_public_key *pk );
 const char *datestr_from_pk( PKT_public_key *pk );
